#import "BezierView.h"

static	const float ptn1 [] = { 5, 0 };
static	const float	ptn2 [] = { 5, 5 };
static	const float	ptn3 [] = { 5, 2 };
static	const float	ptn4 [] = { 3, 3 };
static	const float	ptn5 [] = { 2, 5 };
static	const float	ptn6 [] = { 7, 2, 7 };

struct LineStyleDef	defs [] =
{
	{ ptn1, 2 },
	{ ptn2, 2 },
	{ ptn3, 2 },
	{ ptn4, 2 },
	{ ptn5, 2 },
	{ ptn6, 3 }
};

float	pointDistance ( NSPoint p1, NSPoint p2 )	// Manhattan distance
{
	return fabs ( p1.x - p2.x ) + fabs ( p1.y - p2.y );
}

@implementation BezierView

- (id)initWithFrame:(NSRect)frameRect
{
	if ((self = [super initWithFrame:frameRect]) != nil)
	{
		// Add initialization code here
		lineWidth    = 0.1;
		color        = [[NSColor blackColor] copy];
		index        = -1;
	}
	return self;
}

- (void) dealloc 
{
	[color release];
	[super dealloc];
}

- (void) awakeFromNib
{
	int	i;
	
	[self setLineSize: slider];
	[self setColor: colorWell];
	
	for ( i = 0; i < 6; i++ )
	{
		id <NSMenuItem> item = [styleButton itemAtIndex: i];
		NSString * imageName = [NSString stringWithFormat: @"style-%d", i + 1 ];
		NSString * path      = [[NSBundle mainBundle] pathForResource: imageName ofType: @"tiff"];
		NSImage  * image     = [[NSImage alloc] initWithContentsOfFile: path];
		
		NSLog ( @"%@", image );
		
		[item setImage: image];
	}
}

- (void) drawRect: (NSRect) rect
{
	NSBezierPath * path = [NSBezierPath bezierPath];
	float		   r = 5;
	
	[path moveToPoint: p [0]];
	[path curveToPoint: p [1] controlPoint1: c [0] controlPoint2: c [1]];
	[path setLineWidth: lineWidth];
	[color set];
	[path setLineDash: defs [lineStyle].pattern count: defs [lineStyle].count phase: 0.0];
	[path stroke];
	
	[[NSColor cyanColor] set];
	[self drawPoint: p [0] radius: r];
	[self drawPoint: p [1] radius: r];
	
	[[NSColor yellowColor] set];
	[self drawPoint: c [0] radius: r];
	[self drawPoint: c [1] radius: r];
	
	path = [NSBezierPath bezierPath];
	
	[[NSColor blackColor] set];
	[path moveToPoint: p [0]];
	[path lineToPoint: c [0]];
	[path moveToPoint: p [1]];
	[path lineToPoint: c [1]];
	[path setLineWidth: 0.5];
	[path stroke];
}

- (void) mouseDown : (NSEvent *) event
{
	NSPoint	loc  = [event locationInWindow];
	BOOL	ctrl = ([event modifierFlags] & NSControlKeyMask  ) != 0;
	BOOL	alt  = ([event modifierFlags] & NSAlternateKeyMask) != 0;
	
	loc = [self convertPoint: loc fromView: nil];

	if ( !ctrl && !alt )
	{
		p [0]  = loc;
		p [1]  = loc;
		c [0]  = loc;
		c [1]  = loc;
		index  = -1;
		mode   = modeCreate;
	}
	else
	if ( ctrl )						// we're moving control points
	{
		index = [self indexOfPoint: loc inArray: c];
		mode  = modeDragControl;
	}
	else
	if ( alt )
	{
		index = [self indexOfPoint: loc inArray: p];
		mode  = modeDragPoint;
	}
	else
	{
		index = -1;
		mode  = modeNone;
	}
	
	[self setNeedsDisplay: YES];
}

- (void) mouseDragged: (NSEvent *) event
{
	NSPoint	loc  = [event locationInWindow];
	
	loc = [self convertPoint: loc fromView: nil];

	if ( mode == modeDragControl && index >= 0 )
		c [index] = loc;
	else
	if ( mode == modeDragPoint && index >= 0 )
		p [index] = loc;
	else
	if ( mode == modeCreate )
	{
		p [1] = loc;
		c [1] = loc;
	}
	
	[self setNeedsDisplay: YES];
}

- (void) mouseUp : (NSEvent *) event
{
	index = -1;
}

- (IBAction) setColor: (id) sender
{
	[color autorelease];
	
	color = [[sender color] copy];
	
	[self setNeedsDisplay: YES];
}

- (IBAction) setLineSize: (id) sender
{
	lineWidth = [sender floatValue];
	
	[self setNeedsDisplay: YES];
}

- (int) indexOfPoint: (NSPoint) pt inArray: (NSPoint *) arr
{
	int		  i;

	for ( i = 0; i < 2; i++ )
		if ( pointDistance ( arr [i], pt ) < 10 )
			return i;

	return -1;
}

- (void)    drawPoint: (NSPoint) pt radius: (float) r
{
	[[NSBezierPath bezierPathWithOvalInRect: NSMakeRect ( pt.x - r,  pt.y - r, 2*r, 2*r )] fill];
}

- (IBAction) setLineStyle: (id) sender
{
	lineStyle = [sender indexOfSelectedItem];
	
	[self setNeedsDisplay: YES];
}

@end
